/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the OX Software GmbH group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016-2020 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.guard.mail.service.impl;

import javax.mail.Address;
import javax.mail.Message;
import javax.mail.internet.MimeMessage;
import com.openexchange.exception.OXException;
import com.openexchange.guard.api.GuardApis;
import com.openexchange.guard.internal.AbstractGuardAccess;
import com.openexchange.guard.internal.GuardApiImpl;
import com.openexchange.guard.internal.GuardResult;
import com.openexchange.guard.internal.authentication.GuardAuthenticationToken;
import com.openexchange.guard.mail.service.dataObjects.CryptoSecurityResult;
import com.openexchange.mail.MailField;
import com.openexchange.mail.dataobjects.MailMessage;
import com.openexchange.mail.dataobjects.SecurityResult.EncryptionType;
import com.openexchange.mail.dataobjects.SecuritySettings;
import com.openexchange.mail.mime.converters.MimeMessageConverter;
import com.openexchange.session.Session;

/**
 * {@link GuardMimeVerifier}
 *
 * @author <a href="mailto:greg.hill@open-xchange.com">Greg Hill</a>
 * @since v7.8.4
 */
class GuardMimeVerifier extends AbstractGuardAccess {

    private static final Address[] NO_RECIPIENTS = new Address[] {};
    private static final GuardAuthenticationToken NULL_AUTHENTICATION_TOKEN = null;

    /**
     * Verify signature of Mail Message
     * @param session
     * @param message
     * @return
     * @throws OXException
     */
    public MailMessage verify(Session session, MailMessage message) throws OXException {

        Message originalMimeMessage = MimeMessageConverter.convertMailMessage(message, true);

        Message verifiedMimeMessage = verify(session, originalMimeMessage);

        final MailField[] ALL_FIELDS = {MailField.FULL};
        MailMessage ret = MimeMessageConverter.convertMessage((MimeMessage)verifiedMimeMessage ,
            message.getMailId(),
            message.getFolder(),
            com.openexchange.mail.MailPath.SEPERATOR,
            ALL_FIELDS);
        CryptoSecurityResult security;
        if (message.hasSecurityResult()) {  // If already decoded, just add signature results
            security = (CryptoSecurityResult) message.getSecurityResult();
            security.addHeaderSignatureResults(ret.getHeader("X-Guard-Signature-Result"));
        } else {
            security = new CryptoSecurityResult(ret, EncryptionType.PGP);
        }
        ret.setSecurityResult(security);
        return ret;

    }


    private MimeMessage verify(Session session, Message message) throws OXException {

        GuardApiImpl guardApi = requireGuardApi(PGP_MAIL_ENDPOINT);
        SecuritySettings settings = SecuritySettings.builder().decrypt(true).build();

        GuardResult result = guardApi.processMimeMessage(message, NO_RECIPIENTS, settings,GuardApis.mapFor(
            "action", "verify",
            "user", Integer.toString(session.getUserId()),
            "context", Integer.toString(session.getContextId())),
            session,
            NULL_AUTHENTICATION_TOKEN);
        return result.getMimeMessage();
    }
}
