/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the OX Software GmbH group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016-2020 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.guard.osgi;

import com.openexchange.capabilities.CapabilityService;
import com.openexchange.guard.guest.MailGuestService;
import com.openexchange.guard.guest.impl.GuardGuestFolderHandlerModuleExt;
import com.openexchange.guard.guest.impl.GuardGuestUserService;
import com.openexchange.guard.guest.impl.GuardMailStorageHandler;
import com.openexchange.guard.mail.service.impl.CompositePGPMailRecognizer;
import com.openexchange.guard.mail.service.impl.EncryptedMailServiceImpl;
import com.openexchange.guard.mail.service.impl.GuardAwareMailAccessFactory;
import com.openexchange.guard.mail.service.impl.PGPInlineMailRecognizer;
import com.openexchange.guard.mail.service.impl.PGPMimeMailRecognizer;
import com.openexchange.guard.mail.service.impl.guests.GuardGuestMailProvider;
import com.openexchange.mail.MailProviderRegistry;
import com.openexchange.mail.api.crypto.CryptographicAwareMailAccessFactory;
import com.openexchange.mail.mime.crypto.PGPMailRecognizer;
import com.openexchange.mail.service.EncryptedMailService;
import com.openexchange.mailaccount.MailAccountStorageService;
import com.openexchange.osgi.HousekeepingActivator;
import com.openexchange.share.core.ModuleHandler;
import com.openexchange.share.groupware.spi.FolderHandlerModuleExtension;

/**
 * {@link MailActivator}
 *
 * @author <a href="mailto:greg.hill@open-xchange.com">Greg Hill</a>
 * @since v2.8.0
 */
public class MailActivator extends HousekeepingActivator {

    private static final String GUARD_GUEST_PROTOCOL = null;

    @Override
    protected Class<?>[] getNeededServices() {
        return new Class<?>[] { MailGuestService.class, CapabilityService.class };  // Require MailGuestService to verify GuardActivator loaded
    }

    @Override
    protected void startBundle() throws Exception {

        //For testing purpose:
        GuardGuestMailProvider guardGuestMailProvider = new GuardGuestMailProvider();
        MailProviderRegistry.registerMailProvider(guardGuestMailProvider.getProtocol().getName(), guardGuestMailProvider);

        registerService(EncryptedMailService.class, new EncryptedMailServiceImpl());

        PGPMailRecognizer recognizer = new CompositePGPMailRecognizer(
            new PGPMailRecognizer[] {
                new PGPMimeMailRecognizer(),
                new PGPInlineMailRecognizer()
            });
        registerService(PGPMailRecognizer.class, recognizer);

        registerService(ModuleHandler.class,
            new GuardMailStorageHandler(
                Services.getService(MailAccountStorageService.class),
                new GuardGuestUserService()));

        registerService(CryptographicAwareMailAccessFactory.class, new GuardAwareMailAccessFactory(recognizer, Services.getService(CapabilityService.class)));

        registerService(FolderHandlerModuleExtension.class, new GuardGuestFolderHandlerModuleExt());

        openTrackers();
    }

}
